/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2011-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.opt.oo.light;

import java.io.IOException;
import java.util.List;

import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.juliac.api.ClassGeneratorItf;
import org.objectweb.fractal.juliac.desc.ControllerDesc;
import org.objectweb.fractal.juliac.desc.MembraneDesc;
import org.ow2.frascati.tinfi.control.content.IllegalContentClassMetaData;

/**
 * This class generates the source code associated to Fractal components.
 * The membrane implementation and the initializer implementation are generated.
 * 
 * The content, the interceptors and the controllers are kept in separate
 * classes and the controllers are implemented with objects.
 * 
 * This source code generator uses versions of the Tinfi control membranes that
 * do not provide intent controllers and interceptors.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.4.3
 */
public class FCOOCtrlSourceCodeGenerator
extends org.ow2.frascati.tinfi.opt.oo.FCOOCtrlSourceCodeGenerator {
    
    @Override
    protected void postInit() throws IOException {
    	super.postInit();
    	mloader.put(SCALightPrimitive.NAME,SCALightPrimitive.class);
    	mloader.put(SCALightComposite.NAME,SCALightComposite.class);
    }

    /**
     * Generate the implementation of the membrane associated to a Fractal
     * component.
     * 
     * @param ct                the component type
     * @param ctrldesc          the component controller descriptor
     * @param contentClassName  the name of the component content class
     * @return                  data about the membrane implementation
     */
    @Override
    public MembraneDesc<List<ControllerDesc>> generateMembraneImpl(
        ComponentType ct, String ctrldesc, String contentClassName )
    throws IOException {
    	
        /*
         * Generate the implementation of the SCA Content Controller for the
         * specified content class.
         */
        generateSCAContentControllerImpl(contentClassName);
        
        return super.generateMembraneImpl(ct,ctrldesc,contentClassName);
    }

    @Override
    public InitializerOOCtrlClassGenerator getInitializerClassGenerator() {
        return new InitializerOOCtrlClassGenerator();
    }

    
    // -----------------------------------------------------------------------
    // Implementation specific
    // -----------------------------------------------------------------------

    private void generateSCAContentControllerImpl( String contentClassName )
    throws IOException {
    	
    	if( contentClassName == null ) {
    		return;
    	}
    	
        try {
        	ClassGeneratorItf cg =
        		SCAContentControllerClassGenerator.get(jc,contentClassName);
            jc.generateSourceCode(cg);
        }
        catch( IllegalContentClassMetaData iccmd ) {
        	IOException ioe = new IOException();
        	ioe.initCause(iccmd);
        	throw ioe;
        }
    }    
}
