/***
 * OW2 FraSCAti Tinfi
 * Copyright (C) 2010-2012 Inria, Univ. Lille 1
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: frascati@ow2.org
 *
 * Author: Lionel Seinturier
 */

package org.ow2.frascati.tinfi.tinfilet;

import java.io.IOException;
import java.lang.reflect.Modifier;
import java.util.Date;

import org.objectweb.fractal.juliac.api.ClassGeneratorItf;
import org.objectweb.fractal.juliac.visit.ClassSourceCodeVisitor;
import org.objectweb.fractal.juliac.visit.FileSourceCodeVisitor;

/**
 * Generate an empty class which extends the one specified in the constructor.
 * This generator is used by Tinfi.
 * 
 * @author Lionel Seinturier <Lionel.Seinturier@univ-lille1.fr>
 * @since 1.3.1
 */
public class EmptySubClassGenerator implements ClassGeneratorItf {

	/**
	 * @param targetname   the fully-qualified name of the class to be generated 
	 * @param superclname  the fully-qualified name of the super class
	 */
	public EmptySubClassGenerator( String targetname, String superclname ) {
		this.targetname = targetname;
		this.superclname = superclname;
	}
	
	private String targetname;
	private String superclname;
	
    /**
     * Return the name of the class generated by this generator.
     */
    public String getTargetClassName() {
    	return targetname;
    }

    /**
     * Generate the header of the source code file associated with this
     * generator. The header is the section which precedes the source code of
     * the class. The header may be composed of an initial comment, the
     * declaration of the package name and some import clauses.
     * 
     * @param fv  the visitor for generating the header of the source code file
     */
    public void generate( FileSourceCodeVisitor fv ) throws IOException {
        
    	int lastdot = targetname.lastIndexOf('.');
        String packagename = (lastdot==-1) ? null : targetname.substring(0,lastdot);
        
        /*
         * File header.
         */
        fv.visitFileHeader("/*");
        fv.visitFileHeader(" * Generated by "+getClass().getName()+" on: "+new Date());
        fv.visitFileHeader(" */");
        
        /*
         * Package.
         */
        if( packagename != null ) {
            fv.visitPackageName(packagename);
        }

        /*
         * Class.
         */
        ClassSourceCodeVisitor cv = fv.visitPublicClass();
        generate(cv);    	
    }
    
    /**
     * Generate the source code of the class associated with this generator.
     * 
     * @param cv  the visitor for generating the source code of the class
     */
    public void generate( ClassSourceCodeVisitor cv ) throws IOException {
    	
        int lastdot = targetname.lastIndexOf('.');
        String classname = targetname.substring(lastdot+1);
        cv.visit( Modifier.PUBLIC, classname, null, superclname, null );
        cv.visitEnd();
    }
}
